//-----------------------------------------------------------------------------
// MK3_2560_LOGVIEW
// Version 1.1
// 21.01.2013 / UB
// "www.Mikrocontroller-4U.de"
// mc-4u(@)t-online.de
//-----------------------------------------------------------------------------
// Fuer das myAVR MK3 Board
// mit ATMega-2560 @ 16MHz
// Datenauswertung mit dem PC-Programm "Logview.exe"
//-----------------------------------------------------------------------------
// das File muss im Ordner
// "myAVR/WinAVR/AVR/Include/UB_MK3_2560/"
// liegen und mit #include <UB_MK3_2560/MK3_2560_LOGVIEW.h>
// eingebunden werden
//-----------------------------------------------------------------------------
// Belegt  : nichts (auser einem COM-Port)
// Benutzt : V24-LIB : MK3_2560_V24.h
//
// Hinweis : um die Float-Formatierung zu aktivieren :
//           [beim myAVR-Workpad]
//               muss im Main-Programmkopf diese Zeile stehen :
//               // LinkerOption:    -Wl,-u,vfprintf -lprintf_flt -lm
//           [beim AVR-Studio 4.0]
//               diese Librarys hinzufuegen :
//                "libprintf_flt.a" und "libm.a"
//               und unter "Custom Options" diese Zeile
//               bei den Linker-Options hinzufuegen :
//                "-Wl,-u,vfprintf"
//                
//
//           Das INI-File vom Logview.exe Programm muss zur Initialisierung
//           des MK3-Treibers passen :
//           Es muessen bei beiden die gleiche Anzahl von Kanaelen
//           eingestellt sein
// 
//-----------------------------------------------------------------------------






//----------------------------------------------------------------------------
// Includes
//----------------------------------------------------------------------------
#include "ub_mk3_2560/MK3_2560_V24.h" //  bindet die V24-Funktionen ein
#include <stdio.h>
#include <string.h>



//-----------------------------------------------------------------------------
// Funktionen die vom MAIN benutzt werden koennen
//-----------------------------------------------------------------------------
void MK3_INIT_LOGVIEW(COMPORT port, BAUDRATE baud,unsigned int anzahl);  // zum init des Treibers
void MK3_LOGVIEW_SEND(void);                         // sendet ein Frame
//-----------------------------------------------------------------------------



//----------------------------------------------------------------------------
// Defines
//----------------------------------------------------------------------------

#define LOG_CH_MAXANZ     8          // Maximum Anzahl der Logger Kanaele
#define LOG_MAXWERT      999999.900  // Maximaler Wert (wegen Stringpuffer)
#define LOG_MINWERT     -999999.900  // Minimaler Wert (wegen Stringpuffer)
#define LOG_STRLEN        12         // Stringlaenge eines Wertes
                                     // (Sign+6Stellen+Punkt+3Nachkomma+0x00) 


//-----------------------------------------------------------------------------
// Globale Variablen
//-----------------------------------------------------------------------------

// Laenge des Sende-Puffers (Abhngig von der Kanalanzahl und der Stringlaenge)
#define TX_PUFFER_LEN 10+(LOG_CH_MAXANZ*LOG_STRLEN)


// Structur von Logview
typedef struct {
  char tx[TX_PUFFER_LEN+1];  // RS232 TX-Puffer
  COMPORT com;               // Comport
  unsigned int ch_anz;       // Anzahl der Kanaele
  double ch[LOG_CH_MAXANZ];  // Kanaele als Double Wert
}Logview_t;
Logview_t Logview;



//-----------------------------------------------------------------------------
// Initialisiert den Treiber
// mit einer RS232 Einstellung
// und der Anzahl der Logger Kanaele
//-----------------------------------------------------------------------------
void MK3_INIT_LOGVIEW(COMPORT port, BAUDRATE baud,unsigned int anzahl) {
  unsigned int n;

  // Init der V24-Schnittstelle
  Logview.com=port;
  MK3_INIT_V24(port,baud);

  // Anzahl der Logger-Kanaele setzen
  if(anzahl>LOG_CH_MAXANZ) anzahl=LOG_CH_MAXANZ;
  if(anzahl<1) anzahl=1;
  Logview.ch_anz=anzahl;

  // Logger-Werte zuruecksetzen
  for(n=0;n<Logview.ch_anz;n++) {
    Logview.ch[n]=0;
  }
}


//-----------------------------------------------------------------------------
// Sendet eine Frame per RS232
//-----------------------------------------------------------------------------
void MK3_LOGVIEW_SEND(void) {
  char buf[LOG_STRLEN];
  unsigned int n;
  double wert;

  // Startkennung vom Frame
  strcpy(Logview.tx,"$1;1;;"); // State=1, Timestamp=keine
  // alle Kanalwerte in Strings umwandeln
  for(n=0;n<Logview.ch_anz;n++) {
    // Grenzen pruefen
    wert=Logview.ch[n];
    if(wert>LOG_MAXWERT) wert=LOG_MAXWERT;
    if(wert<LOG_MINWERT) wert=LOG_MINWERT;
    // Zahl umwandeln in string mit 3 Nachkommastellen
    sprintf(buf,"%.3f",wert);
    // zahl anhaengen
    strcat(Logview.tx,buf);
    // Trennungsbyte
    strcat(Logview.tx,";");
  }
  // Endekennung vom Frame
  strcat(Logview.tx,"0");     // CRC=keine
  // Frame senden mit CR+LF
  MK3_V24_OUT(Logview.com,Logview.tx,RX_END_CRLF);
}


